; ----------------------------------------------------
; Adc3.asm    Last Update:10/27/91
;
; Simple Interrupt-Driven Analog-to-Digital
; Converter Example.
; The display is refreshed by the interrupt service
; routine invoked upon Timer0 overflow.
;
; Shows analogue input 0 (AN0) in hexadecimal
; on the 2 digital displays
;
; The voltage seen at the microcontroller pin
; is displayed in hexadecimal (0..ff), 00=0
; volts, and FF=5 volts.  The jumpers jp2
; (vgnd) and jp3 (vref) should be installed.
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; UIOD Connections:
;  LOW DIGIT to Port 4 and HIGH DIGIT to Port 5.
;  80535 port AN0 to UIOD potentiometer AN0.
;
; internal memory use
; bytes:
   Analog0 equ 7Fh     ; store 0-th adc input
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 break     equ  010ch  ; a, (reads accumulator)
 display   equ  011bh  ; a
 inkey     equ  012ah  ; a
 setintvec equ  0145h  ; a, dptr
; ----------------------------------------------------
    org 8000h

    lcall initAdc    ; initialize ADC
    lcall initTim0   ; initialize Timer0
    mov   IP0, #2    ; timer interrupt higher priority
    setb  eal        ; master interrupt enable flag

begin:
    lcall inkey        ; see if there is a CTL-C pending
                       ; note that this is the ONLY
                       ; foreground task!...
    cjne  a, #3, begin ; if not repeat

    clr   eal          ; if CTL-C, clear master interrupt
                       ; enable flag
    clr   TR0          ; Turn timer0 off
    lcall break        ; quit

; ====================================================
;  subroutine initadc - initialize adc
;  input    : none
;  output   : initializes adc interrupt
;             to continuously read analog 0
;  destroys : a, dptr
; ----------------------------------------------------
initAdc:
        clr   eadc        ; disable adc interrupts
        clr   iadc        ; reset adc interrupt flag
        mov   a, #6       ; sixth interrupt source
        mov   dptr, #adcisr ; setup interrupt vector
        lcall setintvec
        setb  EADC        ; ADC interrupt mask bit

        mov   a, ADCON    ; read adc control register
        anl   a, #0d0h    ; select channel 0, single shot
        mov   ADCON, a
        mov   dapr, #0    ; start conversion
        ret

; ====================================================
;  subroutine adcisr
;  input    : adc interrupt
;  output   : continuously obtains and stores analog 0
;             value in internal ram.
;  destroys : nothing - uses a, r0, and r1
; ----------------------------------------------------
adcisr:
    push  psw
    push  acc

    mov   a,   addat     ; read analog value
    mov   Analog0, a     ; store analog value
    mov   a, adcon       ; ADC control register
    anl   a, #0d0h
    mov   adcon, a       ; select channel 0
    clr   iadc           ; reset adc interrupt flag
    mov   dapr, #0       ; start next conversion

    pop   acc
    pop   psw
    reti

; ====================================================
;  subroutine initTim0 - initialize TImer0
;  installs the interrupt service routine
;  Tim0ISR to be invoked upon timer overflow
;
;  input    : none
;  output   : initializes Timer0 in mode 1
;  destroys : a, dptr
; ----------------------------------------------------
initTim0:
      mov   TMOD, #81h     ; configure timer/counter 0
                           ;  timer in mode 1
      mov   TL0, #0        ; start timer from 0
      mov   TH0, #0        ; start timer from 0
      mov   a, #1          ; first interrupt source
      mov   dptr, #Tim0isr ; setup interrupt vector
      lcall setintvec
      setb  ET0            ; Timer0 interrupt mask bit
      setb  TR0            ; turn timer/counter on
      ret

; ====================================================
;  subroutine Tim0isr
;  input    : Timer0 overflow interrupt (TF0)
;  output   : The value Analog0 is displayed on
;             the seven-segment displays.
;
;  destroys : nothing - uses a
; ----------------------------------------------------
Tim0isr:
    push  psw
    push  acc

    mov   a, Analog0   ; get Analog0 value
    push  acc          ; save Analog0 on stack
    anl   a, #0fh      ; isolate the low nibble
    lcall display      ; convert to display pattern
    mov   P4, a        ; display low nibble on LOW DIGIT
    pop   acc          ; recall Analog0
    swap  a            ; swap nibbles
    anl   a, #0fh      ; isolate low nibble
    lcall display      ; get display pattern
    mov   P5, a        ; display on HIGH DIGIT

    pop   acc
    pop   psw
    reti

; ====================================================
;
; end of program
