; ----------------------------------------------------
; DUTY.ASM          Last Update:10/27/92
;
;  Duty Cycle measurement using Timer 2 Capture.
;
;  Both the Pulse and Period is measured using
;  the Timer 2 Capture function.  The duty cycle
;  is the ration of the pulse to the period.
;
;  See comments in program PULSE.ASM for details on
;  Pulse and Period Measurement.
;
;  The program asks for a key input from the host and
;  continues until a Control-C is received.
;
; ====================================================
; UIOD connections
;
;  Connect P1.0 to the input TTL-level pulse.
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; constants
; internal registers:
; --- registers used in 32-bit division ---

  X0   equ   69h
  X1   equ   6Ah
  X2   equ   6Bh
  X3   equ   6Ch

  Y0   equ   6Dh
  Y1   equ   6Eh
  Y2   equ   6Fh
  Y3   equ   70h

  Z0   equ   71h
  Z1   equ   72h
  Z2   equ   73h
  Z3   equ   74h
  Z4   equ   75h
  Z5   equ   76h

  PR0  equ   77h
  PR1  equ   78h
  PR2  equ   79h
  PR3  equ   7Ah

; bits
  ZOV  equ   13h

; --- registers used by pulse/period measurement ---

  BeginL       equ  7Bh   ; beginning count low  byte
  BeginH       equ  7Ch   ; beginning count high byte
  EndL         equ  7Dh   ; end count low  byte
  EndH         equ  7Fh   ; end count high byte
; bits
  SignalHigh   equ  10h   ; set at 0-to-1 transition
                          ; and cleared at 1-to-0
  Trigger      equ  11h   ; set to start captures
  MeasurePulse equ  12h   ; set if pulse measurement
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 crlf      equ  0115h  ; a
 getchr    equ  0121h  ; a
 os_return equ  0130h
 percent   equ  0133h  ; a
 print     equ  0136h  ; a, dptr
 prsphx    equ  0139h  ; a, r2
 prtstr    equ  013ch  ; a
 prthex    equ  013fh  ; a, r2
 setintvec equ  0145h  ; a, dptr
; ----------------------------------------------------
; --- port definitions -------------------------------
; output ports
  PulseIn  equ   90h   ; port bit P1.0
; ----------------------------------------------------
; --- origin at the beginning of RAM -----------------
   org   8000h
; --- program starts here ----------------------------

    lcall print
    db "begin initialization...", 10,13,0

    mov   T2CON, #01010000b ; initialize Timer 2
                            ;  T2I1=0 and T2I0=0
                            ;  T2CM=0
                            ;  T2R1=1 and T2R0=0
                            ;  I3FR=1 (capture upon 0-to-1)
                            ;  I2FR=0
                            ;  T2PS=0
    mov   CCEN, #01         ; CRC capture upon external signal

    mov   a, #5             ; T2 interrupt is the fifth source
    mov   dptr, #T2Isr      ; interrupt service routine for T2
    lcall setintvec         ; set interrupt vector

    mov   a, #8             ; EX3 interrupt is the eigth source
    mov   dptr, #EX3Isr     ; interrupt service routine for EX3
    lcall setintvec         ; set interrupt vector

    setb  ET2               ; allow T2 interrupts
    setb  EX3               ; allow EX3 interrupts
    clr   Trigger


    lcall print
    db "...initialization done", 10,13,0

loop:                       ; each iteration makes a pulse width measurement
    clr   MeasurePulse
    lcall Measure           ; measure period
    mov   Y0, r0            ; save period as the 16-bit divisor
    mov   Y1, r1
    lcall ShowResultsX      ; display measurement results on Host

    setb  MeasurePulse
    lcall Measure           ; measure pulse
    mov   X2, r0            ; save the pulse width asthe dividend
    mov   X3, r1            ; note that the 16-bit pulse width is...
    mov   X1, #0            ; ...stored as the high bytes of the...
    mov   X0, #0            ; ... 32-bit dividend
    lcall ShowResultsX      ; display measurement results on Host
    lcall crlf
    lcall crlf

    lcall DIV32
    jnb   ZOV, NoError
    lcall print
    db   "Error! Division by Zero ",0
    lcall crlf
    sjmp  Ask

NoError:
    lcall print
    db   "Division Result : ",0
    mov   a, Z3
    lcall prsphx
    mov   a, Z2
    lcall prsphx
    lcall crlf
    lcall crlf
    lcall print
    db   "Duty Cycle : ",0
    mov   a, Z3
    lcall percent
    lcall prsphx
    lcall print
    db    "% ",10,13,10,13,0

; --- ask host whether to continue or quit ----------------------------
Ask:
    lcall print
    db   "press any key to continue or Ctl-C to quit...", 0
    lcall crlf
    lcall crlf
    lcall getchr
    cjne  a, #3, Again
    lcall print             ; print message
    db    10,13, "CTL-C received - program terminates", 10,13,0
    clr   ET2               ; disable T2 interrupts
    clr   EX3               ; disable EX3 interrupts
    ljmp  os_return         ; return to OS

Again:
    ljmp loop
; ====================================================
; --- include files ----------------------------------
#include DutyMath.inc
; ====================================================

; ====================================================
;  subroutine T2Isr
;
;  TF2 must be cleared by software
;
;  input    : Timer 2 overflow
;  output   : nothing
;  destroys : nothing
; ----------------------------------------------------
T2Isr:
     clr   TF2           ; clear overflow flag
     reti


; ====================================================
;  subroutine Ex3Isr
;
;  External Interrupt 3
;
;  input    : External Interrupt 3
;  output   : updates registers BeginL, BeginH, EndL,
;             EndH, and flag SignalHigh
;  destroys : nothing
; ----------------------------------------------------
EX3Isr:
     jnb   Trigger, EX3done
     jb    SignalHigh, SH
     mov   BeginL, CRCL
     mov   BeginH, CRCH
     mov   CRCL, #0
     mov   CRCH, #0
     setb  SignalHigh
     jnb   MeasurePulse, EX3done
     clr   I3FR            ; next capture upon 1-to-0
     reti
SH:
     mov   EndL, CRCL
     mov   EndH, CRCH
     mov   CRCL, #0
     mov   CRCH, #0
     clr   SignalHigh
     clr   Trigger
     setb  I3FR
EX3done:
     reti

; ====================================================
;  subroutine Measure
;
;  Performs a singel pulse (or period) measurement
;
;  input    : none
;  output   : Pulse Width in R1 and R0; begin count in
;             registers BeginL and BeginH; end count in
;             EndL and EndH.
;  destroys : a, r2, dptr
; ----------------------------------------------------
Measure:
    clr   SignalHigh        ; initialize pulse flag
    mov   CRCL, #0
    mov   CRCH, #0
    setb  EAL               ; allow interrupts
    setb  T2I0              ; start Timer 2
    setb  Trigger           ; trigger measurement
here:
    jb    Trigger, Here     ; wait until pulse is measured
    clr   T2I0              ; stop Timer 2
    clr   EAL               ; disable all interrupts
    clr   TF2               ; clear timer 2 overflow flag - just in case...
; --- the pulse width measurement is complete.  The end and begin -----
; --- counts are stored in registers EndL, EndH, BeginL, and BeginH ---
; --- Note that the timer and the interrupts are disabled -------------

    clr   C                 ; compute pulse width
    mov   a, EndL
    subb  a, BeginL
    mov   r0, a             ; save pulse width low byte in r0
    mov   a, EndH
    subb  a, BeginH
    mov   r1, a             ; save pulse width low byte in r1
    ret


; ====================================================
;  subroutine ShowResultsX
;
;  Displays pulse measurement results
;
;  input    : Pulse Width in R1 and R0; begin count in
;             registers BeginL and BeginH; end count in
;             EndL and EndH.
;             Also inspects flag MeasurePulse to determine
;             the message (pulse or period)
;  output   : writes messages to host screen
;  destroys : a, r2, dptr
; ----------------------------------------------------
ShowResultsX:
    jnb   MeasurePulse, MPeriod
    lcall print             ; print pulse width
    db   "pulse width  : ", 0
    sjmp  SR_1
MPeriod:
    lcall print             ; print pulse width
    db   "period width : ", 0
SR_1:
    mov   a, r1
    lcall prsphx
    mov   a, r0
    lcall prsphx
    lcall crlf

    lcall print             ; print Timer2 count at pulse end
    db   "end count    : ", 0
    mov   a, EndH
    lcall prsphx
    mov   a, EndL
    lcall prsphx
    lcall crlf

    lcall print             ; print Timer2 count at pulse start
    db   "begin count  : ", 0
    mov   a, BeginH
    lcall prsphx
    mov   a, BeginL
    lcall prsphx
    lcall crlf
    lcall crlf
    ret


; ====================================================
;
; end of program
