; ----------------------------------------------------
; DutyMath.inc      Last Update:10/24/92
;
; Math routines for Computing Duty Cycles
;
; ====================================================

;====================================================================
; subroutine SUB32
; 32-Bit Unsigned subtraction
;
; input:     internal registers X3, X2, X1, X0, and Y3, Y2, Y1, and Y0
;          hold the 32-bit unsigned integers
; output:     internal registers Z3, Z2, Z1, and Z0 hold the result
;          Z=X-Y.  addressible bit ZOV is set if the result (Z) is
;          out of range (if an external barrow is produced)
; destroys:     a, flags
;====================================================================
SUB32:
     mov     a, X0          ; load X low byte into accumulator
     clr     C              ; clear carry flag
     subb    a, Y0          ; subract Y low byte
     mov     Z0, a          ; put result in Z low byte
     mov     a, X1          ; repeat with other bytes...
     subb    a, Y1
     mov     Z1, a
     mov     a, X2
     subb    a, Y2
     mov     Z2, a
     mov     a, X3
     subb    a, Y3
     mov     Z3, a
     mov     ZOV, C         ; set ZOV if an external barrow is produced
     ret                    ; done
;====================================================================
; subroutine DIV32
; 32-Bit Unsigned Division
;
; input:    internal registers X3, X2, X1, and X0 hold the 32-bit
;           unsigned dividend.  similarly, internal registers Y1 and
;           Y0 hold the high and low bytes of the unsigned divisor.
;
; output:   internal registers Z5, Z4, Z3, and Z2 hold the 32-bit
;           quotient of Z=XY.  internal registers Z1 and Z0 hold
;           the high and low bytes of the remainder.  addressible bit
;           ZOV is set if Y=0, i.e., the result is out of range.
;
;           internal registers PR3, PR2, PR1, and PR0 store the
;           partial remainder.  r5, r4, r3, and r2 hold the partially
;           computed quotient.
;
; calls:    SUB16
; destroys: a, r2, r3, r4, r5, r7, flags
;====================================================================
DIV32:
     mov     a, Y1           ; get divisor high byte
     orl     a, Y0           ; OR with low byte
     jnz     div32_OK        ; divisor OK if not 0
     setb    ZOV             ; else, overflow
     ret
div32_OK:
     mov     Y2, #0          ; high 16 bits of Y is 0
     mov     Y3, #0
     mov     PR3, X3         ; store dividend
     mov     PR2, X2
     mov     PR1, X1
     mov     PR0, X0
     mov     X3, #0          ; clear partial remainder
     mov     X2, #0
     mov     X1, #0
     mov     X0, #0
     mov     r6, #0
     mov     r5, #0
     mov     r4, #0          ; clear partial quotient
     mov     r3, #0
     mov     r7, #32         ; set loop count

div32_loop:

     clr     C               ; clear carry flag
     mov     a, PR0          ; shift the highest bit of the dividend...
     rlc     a               ; ... into...
     mov     PR0, a
     mov     a, PR1
     rlc     a
     mov     PR1, a
     mov     a, PR2
     rlc     a
     mov     PR2, a
     mov     a, PR3
     rlc     a
     mov     PR3, a

     mov     a, X0           ; ... the lowest bit of the partial ...
     rlc     a               ; ... remainder
     mov     X0, a
     mov     a, X1
     rlc     a
     mov     X1, a
     mov     a, X2
     rlc     a
     mov     X2, a
     mov     a, X3
     rlc     a
     mov     X3, a

     lcall   SUB32           ; attempt to subtract to see if the...
                             ; ... partial remainder is as large or...
                             ; ... larger than the divisor.

     mov     C, ZOV          ; get subtraction external barrow
     cpl     C               ; complement external barrow
     jnc     div32_1         ; do not update partial remainder if no barrow

     mov     X3, Z3          ; update partial quotient
     mov     X2, Z2
     mov     X1, Z1
     mov     X0, Z0
div32_1:
     mov     a, r3           ; add result bit to partial quotient
     rlc     a
     mov     r3, a

     mov     a, r4
     rlc     a
     mov     r4, a

     mov     a, r5
     rlc     a
     mov     r5, a

     mov     a, r6
     rlc     a
     mov     r6, a

     djnz     r7, div32_loop

     mov     Z5, r6          ; put quotient in Z3, and Z2
     mov     Z4, r5
     mov     Z3, r4
     mov     Z2, r3
     mov     Z1, X1          ; get remainder, saved before the...
     mov     Z0, X0          ; last subtraction
     clr     ZOV             ; divisor is not 0
     ret                     ; done
; ====================================================
;
; end of include module
