; ----------------------------------------------------
; EXP-FBC.ASM          Last Update:10/27/91
; Feedback Control Experiment
;
;  This program controls the speed of a DC motor by
;  Pulse Width Modulation (PWM).  The motor is
;  switched on for a period of time, and then off.  The
;  fraction of time the motor is on is called the duty
;  cycle.  This program uses a byte to store the time the
;  motor is on, i.e., the number of cycles out of 256
;  cycles, while the motor is on.  A value of 10 means
;  that the motor is on 10 cycles and off 246 cycles.
;  The duty cycle value is stored in the internal
;  register labeled dCycle, defined by an EQU pseudo-op.
;  SImilarly, its complement is stored in dCycleC.
;
;  Timer 0 of the 8031/80535 microcontroller is used.
;  Timer 0 is initialized to run in mode 2.  This is a
;  8-bit timer, incremented every machine cycle.
;  Each overflow reloads the high count register into
;  the low count register.  If the reload value is 0,
;  timer 0 overflows every 256 microseconds, and if FF.
;  it overflows every microsecond.
;
;  The program first turns the motor on and places the
;  dCycle value into TH0 as the reload value.  Upon timer
;  overflow, the complement of the duty cycle, dCycleC
;  is placed into TH0 as the reload value and the motor
;  is turned off.
;
;  The motor state is kept in a bit-addressible register,
;  labeled as the motorFlag.  Note that this flag is
;  necessary since the port bit may read 0 when it is
;  driving a low impedance load.
;
;  When the timer overflows, an interrupt is generated.
;  The Interrupt Service Routine (ISR) selects the next
;  reload value depending on the state of the motor read
;  from the motorFlag.  The ISR allows the entire motor
;  drive code to run in the background.  This program
;  implements a foreground task which displays the values
;  of the tachometer and reference voltages and the current
;  duty cycle.
;
;  The duty cycle is changed by the subroutine UPDATE.
;  The Timer0 ISR uses duty cycle to set the reload value,
;  which in turn determines the speed of the motor.
;  UPDATE is called by subroutine ADCISR.  Thus, UPDATE
;  is called periodically.  Each time UPDATE is called,
;  an internal register labeled COUNT is decremented.
;  The duty cycle is updated when COUNT reaches 0.
;  This software counter is to prevent the duty cycle
;  from being changed too rapidly (to prevent
;  overshooting).  If the tachometer voltage is greater
;  than the reference voltage duty cycle is decremented.
;  If tachometer oltage is less than the reference voltage
;  the duty cycle is incremented.  If the tachometer
;  voltage is equal to the reference voltage, then the
;  duty cycle is not changed.  A check is made before the
;  duty cycle is changed to assure that the duty cycle
;  is not incremented beyond its upper bound (DCUB) or
;  decremented below its lower bound (DCLB).  These
;  parameters are defined by EQU pseudo-ops.
;
;
;  See the accompanying circuit diagram for further
;  information.
;
; ====================================================
; UIOD connections
;
;  Connect P1.0 to the switching transistor as given
;  in the accompanying circuit diagram.
;  Connect the regulated tachometer voltage to AN0 and
;  the reference voltage from potentiometer AN0 of UIOD
;  to analog input AN1.
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; constants
  DCLB      equ  0
  DCUB      equ  0F0h

; internal memory use
; bits
  motorFlag equ  20h   ; set if motor is on
  ADCch     equ  21h   ; ADC channel (0 or 1)
; bytes:
   count    equ  7Ah  ; sotware counter low byte
   analog0   equ  7Ch  ; 0-th adc input from tachometer
   analog1   equ  7Dh  ; 1-st adc input from reference
   dCycle    equ  7Eh  ; duty cycle [0..FF]
   dCycleC   equ  7Fh  ; complement of duty cycle
; ----------------------------------------------------
; the jumpers jp2 (vgnd) and jp3 (vref) should be
; installed.
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 inkey     equ  012ah  ; a
 mdelay    equ  012dh  ; a
 os_return equ  0130h
 print     equ  0136h  ; a, dptr
 prsphx    equ  0139h  ; a, r2
 setintvec equ  0145h  ; a, dptr
; ----------------------------------------------------
; --- port definitions -------------------------------
; output ports
  PWM equ   90h   ; port bit P1.0
; ----------------------------------------------------
; --- origin at the beginning of RAM -----------------
    org   8000h
; --- program starts here ----------------------------
    mov   dCycle, #0     ; initialize duty cycle (off)
    mov   dCycleC, #0FFh
    mov   IP0, #2h       ; Timer 0 interrupt has...
    mov   IP1, #0        ; ... highest priority
    mov   IEN0, #0       ; all interrupts disabled
    mov   IEN1, #0
    lcall itim0          ; initialize timer 0
    lcall initadc        ; initialize analog-to-digital conv.
    setb  EAL            ; enable all interrupts
    setb  T0             ; start timer

loop:                    ; this loop is the foreground task

    mov   a, #0          ; delay 256 milliseconds
    lcall mdelay         ; slow down printing to host

    lcall print          ; print tachometer value
    db    10,13,"tach ",0
    mov   a, analog0     ; (analog 0 is tach)
    lcall prsphx

    lcall print          ; print reference value
    db    " ref ",0
    mov   a, analog1     ; (analog 1 is ref)
    lcall prsphx

    lcall print          ; print duty cycle
    db    "   dCycle ",0
    mov   a, dCycle
    lcall prsphx

    lcall inkey          ; check for Control-C
    cjne  a, #3, loop    ; if no CTL-C repeat loop
                         ; else start shut down
    clr   T0             ; turn Timer 0 off
    clr   EAL            ; disable all interrupts
    lcall print          ; print message
    db    10,13, "CTL-C received - program terminates", 10,13,0
    ljmp  os_return      ; return to OS

; ::::::::::::::::::::::::::::::::::::::::::::::::::::
; --- subroutines ------------------------------------

; ====================================================
;  subroutine initialize timer 0
;
;  Timer 0 is initialized to run in mode 2, an 8-bit
;  counter.  Each time it overflows, it generates an
;  interrupt, which is services by routine isrt0.
;  In addition, the high counter register is reloaded
;  into the low count register.  The larger the reload
;  value, the sooner the time to overflow.
;
;  input    : none
;  output   : none
;  destroys : a, dptr
; ----------------------------------------------------
itim0:
    anl   TMOD, #0f0h    ;  initialize timer 0
    orl   TMOD, #2       ;  mode 2

    mov   dptr, #isrt0   ;  isr for timer 0
    mov   a, #1          ;  modify int t0 vector
    lcall setintvec
    mov   TH2, dCycle   ;  initialize software clock
    setb  ET0           ;  enable timer 0 int
    setb  TR0
    ret

; ====================================================
;  subroutine isrt0
;
;  Timer 0 is initialized to run in mode 2.  This is a
;  8-bit timer, incremented every machine cycle.
;  Each overflow reloads the high count register into
;  the low count register.  If the reload value is 0,
;  timer 0 overflows every 256 microseconds, and if FF.
;  it overflows every microsecond.
;
;  The ISR checks the status of the motor by inspecting
;  motorFlag.  If the motor is on, it is turned off -
;  and if the motor is off, it the turned on.  The time
;  the motor is on or off depends on the time to timer
;  overflow, and hence the reload value.  The ISR places
;  either dCycle or its complement (256-dCycle) depending
;  on whether the next cycle is an on cycle or an off
;  cycle.
;
;  Since an interrupt service routine is called
;  irrespective of what the foreground program is
;  executing (e.g. a compare instruction), it should
;  not modify any of the registers or flags.  All
;  registers used by the routine are saved on stack
;  and restored before the reoutine returns.  Moreover,
;  the ISR needs to complete its execution before the
;  next interrupt, that is, before timer overflow.
;  The largest duty cycle is F0h, giving the ISR only
;  16 cycles to execute - including the call and the
;  return.  This is why the complement of dCycle is
;  kept in a register rather that computed by the ISR
;  every time it is needed.  To see what happens when
;  the ISR takes too long, insert a few NOP operations
;  in the beginning of the ISR and specify a high
;  speed.
;
;  input    : timer 0 overflow
;  output   : toggles PWM output and places a reload
;             value into TH0.
;
;  destroys : nothing - uses a
; ----------------------------------------------------
isrt0:
;    nop    ; add a few nop instructions here and see
;    nop    ; what happens when a high speed, e.g. F,
            ; is selected.  See experimenter's guide.

    jb    motorFlag, motorOff ; if motor is on, then turn off
    setb  PWM                 ; else turn motor on
    mov   TH0, dCycle         ; prepare for next count
    setb  motorFlag           ; update motor status flag
    reti

motorOff:
    clr   PWM                 ; turn motor off
    mov   TH0, dCycleC        ; prepare for next count
    clr   motorFlag           ; update motor status flag
    reti

; ====================================================
;  subroutine update - update duty cycle
;  This routine is called by subroutine adcisr.  An
;  internal register labeled COUNT is decremented.
;  When COUNT reaches 0, the duty cycle is updated.
;  The software counter is to prevent the duty cycle
;  from being changed too rapidly - to prevent
;  overshooting.
;
;  input    : none
;  output   : increments or decrements the duty cycle
;  destroys : a and b
; ----------------------------------------------------
update:
    djnz  count, udDone ; update software counter

    mov   a, analog1    ; from reference
    mov   b, analog0    ; from tachometer
    clr   C
    subb  a, b          ; ref-tach
    jnz   update1       ; update if error not zero
    ret                 ; return if tach=reference

update1:
    jc    decrease      ; if tach>ref decrease dCycle
    lcall higherDC      ; else incerease dCycle
    ret
decrease:
    lcall lowerDC       ; if ref>tach increase dCycle
udDone:
    ret

; ====================================================
;  subroutine lowerDC - lower duty cycle
;  duty cycle is decremented provided that it is
;  currently greater than the lower bound DCLB
;  (defined by en EQU pseude-op).
;  input    : none
;  output   : decrements the duty cycle stored in
;             internal register dCycle and increments
;             the complement stored in dCycleC
;  destroys : a
; ----------------------------------------------------
lowerDC:
    mov   a, dCycle       ; read duty cycle
    cjne  a, #DCLB, DecDC ; return if at lower bound
    ret
DecDC:
    dec   dCycle          ; else update dCycle...
    inc   dCycleC         ; ... and dCycleC
    ret

; ====================================================
;  subroutine higherDC - higher duty cycle
;  duty cycle is incremented provided that it is
;  currently less than the upper bound DCUB (defined
;  by en EQU pseude-op).
;  input    : none
;  output   : increments the duty cycle stored in
;             internal register dCycle and decrements
;             the complement stored in dCycleC
;  destroys : a
; ----------------------------------------------------
higherDC:
    mov   a, dCycle       ; read duty cycle
    cjne  a, #DCUB, IncDC ; return if at upper bound
    ret
IncDC:
    inc   dCycle          ; else update dCycle and...
    dec   dCycleC         ; dCycleC.
    ret

; ====================================================
;  subroutine initadc - initialize adc
;  input    : none
;  output   : initializes adc interrupt
;             to continuously read analog 0
;  destroys : a, dptr
; ----------------------------------------------------
initadc:
        clr   eadc        ; disable adc interrupts
        clr   iadc        ; reset adc interrupt flag
        mov   a, #6       ; sixth interrupt source
        mov   dptr, #adcisr ; setup interrupt vector
        lcall setintvec
        setb  eadc        ; adc interrupt mask bit

        mov   a, adcon    ; read adc control register
        anl   a, #0d0h    ; select channel 0, single shot
        mov   adcon, a
        mov   dapr, #0    ; start conversion
        ret

; ====================================================
;  subroutine adcisr
;  input    : adc interrupt
;  output   : continuously obtains and stores analog 0
;             value in internal ram.
;  destroys : nothing - uses a, r0, and r1
; ----------------------------------------------------
adcisr:
    push  psw
    mov   psw, #0        ; select register bank 0
    push  acc
    push  0              ; push register 0

    jb    ADCch, ch1     ; if ADCch=1 then read ch1
                         ; ... else read channel 0
    setb  ADCch          ; toggle channel number
    mov   r0,  #analog0 ; analog 0 store address
    mov   a,   addat     ; read analog value
    mov   @r0, a         ; store analog value
    mov   a, adcon       ; adc control register
    anl   a, #0d0h       ; select channel 1 next
    setb  acc.0          ; 1
    mov   adcon, a       ; select next channel
    clr   iadc           ; reset adc interrupt flag
    mov   dapr, #0       ; start next conversion
    sjmp  adcDone

ch1:
    clr   ADCch          ; toggle channel number
    mov   r0,  #analog1 ; analog 1 store address
    mov   a,   addat     ; read analog value
    mov   @r0, a         ; store analog value
    mov   a, adcon       ; adc control register
    anl   a, #0d0h       ; select channel 0 next
    mov   adcon, a       ; select next channel
    clr   iadc           ; reset adc interrupt flag
    mov   dapr, #0       ; start next conversion

adcDone:
    lcall update
    pop   0
    pop   acc
    pop   psw
    reti

; ====================================================
;
; end of program
