; ----------------------------------------------------
; EXP-PWM.ASM          Last Update:10/27/91
; Digital-to-Analog COnversion by Pulse Width
;  Modulation
;
;  This program controls the speed of a DC motor by
;  Pulse Width Modulation (PWM).  The motor is
;  switched on for a period of time, and then off.  The
;  fraction of time the motor is on is called the duty
;  cycle.  This program uses a byte to store the time the
;  motor is on, i.e., the number of cycles out of 256
;  cycles, while the motor is on.  A value of 10 means
;  that the motor is on 10 cycles and off 246 cycles.
;  The duty cycle value is stored in the internal
;  register labeled dCycle, defined by an EQU pseudo-op.
;
;  Timer 0 of the 8031/80535 microcontroller is used.
;  Timer 0 is initialized to run in mode 2.  This is a
;  8-bit timer, incremented every machine cycle.
;  Each overflow reloads the high count register into
;  the low count register.  If the reload value is 0,
;  timer 0 overflows every 256 microseconds, and if FF.
;  it overflows every microsecond.
;
;  The program first turns the motor on and places the
;  dCycle value into TH0 as the reload value.  Upon timer
;  overflow, the complement of the dCycle value is placed
;  into TH0 as the reload value and the motor is turned off.
;  The motor state is kept in a bit-addressible register,
;  labeled as the motorFlag.  Note that this flag is
;  necessary since the port bit may read 0 when it is
;  driving a low impedance load.
;
;  When the timer overflows, an interrupt is generated.
;  The Interrupt Service Routine (ISR) selects the next
;  reload value depending on the state of the motor read
;  from the motorFlag.  The ISR allows the entire motor
;  drive code to run in the background.  This program
;  implements a foreground task which asks for a speed
;  from the user and stores this information in dCycle
;  The ISR uses this information to set the reload value.
;
;  This experiment illustrates the use of interrupts and
;  setting the interrupt vectors in high memory RAM by
;  the system call setintvec.
;
;  The program uses bit 0 of Port 1 to drive the
;  switching transistor.   See the experiment guide
;  for further information on the circuit.
;
; ====================================================
; UIOD connections
;
;  Connect P1.0 to the switching transistor as given
;  in the accompanying circuit diagram.
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; internal memory use
; bits
  motorFlag equ  20h   ; set if motor is on
; bytes:
  dCycle    equ  7Eh   ; duty cycle [0..FF]
  dCycleC   equ  7Fh   ; complement of duty cycle
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 ascbin    equ  0100h  ; a, r2, error flag
 beep      equ  0106h  ;  none
 getchr    equ  0121h  ; a
 os_return equ  0130h
 print     equ  0136h  ; a, dptr
 setintvec equ  0145h  ; a, dptr
 sndchr    equ  0148h  ; a
;bits
 errorf    equ     3   ; monitor error flag
; ----------------------------------------------------
; --- port definitions -------------------------------
; output ports
  PWM equ   90h   ; port bit P1.0
; ----------------------------------------------------
; --- origin at the beginning of RAM -----------------
   org   8000h
; --- program starts here ----------------------------
    lcall print
    db "about to begin initialization...", 10,13,0

    mov   dCycle, #0     ; initialize duty cycle (off)

    mov   IP0, #0        ; all interrupts low priority
    mov   IP1, #0
    lcall itim0          ; initialize timer 0
    lcall print
    db "...initialization done", 10,13,0
    setb  TR0            ; start timer
    setb  EAL            ; enable all interrupts

loop:                    ; this loop is the foreground task
    lcall print          ; print prompt
    db    10,13,"input new speed (0..F) ", 0
    lcall getchr
    cjne  a, #3, noCtlC
    clr   TR0            ; turn timer 0 off
    clr   EAL            ; disable all interrupts
    lcall print          ; print message
    db    10,13, "CTL-C received - program terminates", 10,13,0
    ljmp  os_return      ; return to OS

noCtlC:
    push  acc            ; save character on stack
    lcall sndchr         ; echo the character to screen
    pop   acc            ; restore character
    clr   errorf
    lcall ascbin         ; convert the character to binary
    jnb   errorf, digitOK   ; see character in 0..F
    lcall beep
    sjmp  loop
digitOK:
    swap  a              ; effectively multiply by 16
    anl   a, #0F0h       ; make sure low nibble is 0
    mov   dCycle, a      ; use as the reload value
    cpl   a              ; complement a
    mov   dCycleC, a     ; store dCycle complement
    sjmp  loop           ; repeat

; ::::::::::::::::::::::::::::::::::::::::::::::::::::
; --- subroutines ------------------------------------
; ====================================================
;  subroutine initialize timer 0
;
;  Timer 0 is initialized to run in mode 2, an 8-bit
;  counter.  Each time it overflows, it generates an
;  interrupt, which is services by routine isrt0.
;  In addition, the high counter register is reloaded
;  into the low count register.  The larger the reload
;  value, the sooner the time to overflow.
;
;  input    : none
;  output   : none
;  destroys : a, dptr
; ----------------------------------------------------
itim0:
    anl   TMOD, #0f0h    ;  initialize timer 0
    orl   TMOD, #2       ;  mode 2

    mov   dptr, #isrt0   ;  isr for timer 0
    mov   a, #1          ;  modify int t0 vector
    lcall setintvec
    mov   TH2, dCycle   ;  initialize software clock
    setb  ET0           ;  enable timer 0 int
    setb  TR0
    ret

; ====================================================
;  subroutine isrt0
;
;  Timer 0 is initialized to run in mode 2.  This is a
;  8-bit timer, incremented every machine cycle.
;  Each overflow reloads the high count register into
;  the low count register.  If the reload value is 0,
;  timer 0 overflows every 256 microseconds, and if FF.
;  it overflows every microsecond.
;
;  The ISR checks the status of the motor by inspecting
;  motorFlag.  If the motor is on, it is turned off -
;  and if the motor is off, it the turned on.  The time
;  the motor is on or off depends on the time to timer
;  overflow, and hence the reload value.  The ISR places
;  either dCycle or its complement (256-dCycle) depending
;  on whether the next cycle is an on cycle or an off
;  cycle.
;
;  Since an interrupt service routine is called
;  irrespective of what the foreground program is
;  executing (e.g. a compare instruction), it should
;  not modify any of the registers or flags.  All
;  registers used by the routine are saved on stack
;  and restored before the reoutine returns.  Moreover,
;  the ISR needs to complete its execution before the
;  next interrupt, that is, before timer overflow.
;  The largest duty cycle is F0h, giving the ISR only
;  16 cycles to execute - including the call and the
;  return.  This is why the complement of dCycle is
;  kept in a register rather that computed by the ISR
;  every time it is needed.  To see what happens when
;  the ISR takes too long, insert a few NOP operations
;  in the beginning of the ISR and specify a high
;  speed.
;
;  input    : timer 0 overflow
;  output   : toggles PWM output and places a reload
;             value into TH0.
;
;  destroys : nothing - uses a
; ----------------------------------------------------
isrt0:
    push  acc

;    nop    ; add a few nop instructions here and see
;    nop    ; what happens when a high speed, e.g. F,
            ; is selected.  See experimenter's guide.

    jb    motorFlag, motorOff ; if motor is on, then turn off
    setb  PWM                 ; else turn motor on
    mov   TH0, dCycle         ; prepare for next count
    setb  motorFlag           ; update motor status flag
    pop   acc
    reti

motorOff:
    clr   PWM                 ; turn motor off
    mov   TH0, dCycleC        ; prepare for next count
    clr   motorFlag           ; update motor status flag
    pop   acc
    reti

; ====================================================
;
; end of program
