; ----------------------------------------------------
; EXP-STP3.ASM          Last Update:10/27/91
; Stepper Motor Control Experiment 3
;
;  This program is similar to EXP-STP1.ASM except it
;  uses Timer 0 of the 8031/80535 microcontroller in
;  mode 0 to generate the delays between successive
;  steps.
;  When the timer overflows, an interrupt is generated.
;  The Interrupt Service Routine (ISR) selects the next
;  step pattern and sends to Port 1.  The ISR allows the
;  entire motor drive code to run in the background.
;
;  This experiment illustrates the use of interrupts and
;  setting the interrupt vectors in high memory RAM by
;  the system call setintvec.
;
;  The program uses the lower nibble of Port 1 to drive
;  the  switching transistors.   See the experiment guide
;  for further information on the circuit.  The
;  program implements the half-step mode.
;
;  This version differs from EXP-STP1.ASM in that the
;  stepping sequence is provided by the subroutine
;  GETSTEP.
;
; ====================================================
; UIOD connections
;
;  Connect P1.0 to P1.3 to the switching transistors
;  as given in the accompanying circuit diagram.
;
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 inkey     equ  012ah  ; a
 os_return equ  0130h
 print     equ  0136h  ; a, dptr
 setintvec equ  0145h  ; a, dptr
; ----------------------------------------------------
; --- internal register use --------------------------
  STEP    equ   07fh   ; store current step
; constants
  COUNT   equ 20h      ; sets delay time - try other
                       ; values for different speeds
; ----------------------------------------------------
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; --- origin at the beginning of RAM -----------------
   org   8000h
; --- program starts here ----------------------------

    lcall itim0          ; initialize timer 0
    setb  TR0            ; start timer
    setb  EAL            ; enable all interrupts

loop:                    ; this loop is the foreground task
    lcall inkey          ; wait for a Control-C
    cjne  a, #3h, loop

; --- loop terminates when Control-C is received -----
    clr   TR0            ; turn timer 0 off
    clr   EAL            ; disable all interrupts
    lcall print          ; print message
    db    10,13, "CTL-C received - program terminates", 10,13,0
    ljmp  os_return      ; return to OS

; ::::::::::::::::::::::::::::::::::::::::::::::::::::
; --- subroutines ------------------------------------
; ====================================================
;  subroutine initialize timer 0
;
;  Timer 0 is initialized to run in mode 0, a 13-bit
;  counter.  Each time it overflows, it generates an
;  interrupt, which is services by routine isrt0.
;
;  input    : none
;  output   : none
;  destroys : a, r0, dptr
; ----------------------------------------------------
itim0:
    anl   TMOD, #0f0h    ;  initialize timer 0

    mov   dptr, #isrt0   ;  isr for timer 0
    mov   a, #1          ;  modify int t0 vector
    lcall setintvec
    mov   step, #0      ;  initialize software clock
    setb  ET0           ;  enable timer 0 int
    setb  TR0
    ret

; ====================================================
;  subroutine isrt0
;
;  Timer 0 is initialized to run in mode 0.  This is a
;  13-bit timer, incremented every machine cycle.
;  thus, timer 0 overflows every 8192 microseconds.
;  Since an interrupt service routine is called
;  irrespective of what the foreground program is
;  executing (e.g. a compare instruction), it should
;  not modify any of the registers or flags.  All
;  registers used by the routine are saved on stack
;  and restored before the reoutine returns.
;
;  input    : timer 0 overflow
;  output   : calls getpattern to obtain the next
;             step pattern and outputs pattern to P1
;
;  destroys : nothing - uses a, r0
; ----------------------------------------------------
isrt0:
    push  acc
    lcall getpattern  ; get next step pattern
    mov   P1, a       ; send pattern to port
    pop   acc
    reti

; ====================================================
;  subroutine getpattern
;  This routine keeps track of the current step [0..7]
;  in internal register STEP.
;
;  Each time getpattern is called it returns the next
;  step pattern in the accumulator.
;
;  input    : none
;  output   : the next step pattern in accumulator
;  destroys : r0, r1
; ----------------------------------------------------
getpattern:
         mov a, STEP  ; get current step
         inc a        ; increment step
         anl a, #7    ; modulus 8
         mov STEP, a  ; store new step

         inc  a       ; to account for the ret instruction
         movc a, @a+pc
         ret
;--- table of bit patterns for steps ---
         db 0Ah, 08h, 09h, 01h, 05h, 04h, 06h, 02h

; ====================================================
;
; end of program
