; ----------------------------------------------------
; KeysPoll.asm       Last Update:10/24/92
;
; Keypad scan routines
;  the polling implementation
;
; Courtesy Rigel Corporation 
;
; ====================================================
; UIOD connections
; output ports
;    Col0      to P1.4
;    Col1      to P1.5
;    Col2      to P1.6
;    Col3      to P1.7
;
; input ports
;    Row0      to P1.0
;    Row1      to P1.1
;    Row2      to P1.2
;    Row3      to P1.3
;
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 crlf      equ  0115h  ; a
 delay     equ  0118h  ; a
 display   equ  011bh  ; a
 getchr    equ  0121h  ; a
 inkey     equ  012ah  ; a
 os_return equ  0130h
 print     equ  0136h  ; a, dptr
 prthex    equ  013fh  ; a, r2
 setintvec equ  0145h  ; a, dptr
; ----------------------------------------------------

; ====================================================
;       m a i n    p r o g r a m
; ====================================================

     org 8000h
start:
poll:
     lcall   KeyPressed      ; see if a key is pressed
     jnc     poll            ; repeat if no key is pressed

; --- if a key is pressed the program falls out of the loop ---
; --- the b register keeps row and column number of the -------
; --- pressed key in its low and high nibbles, respectively. --

     lcall   FindRow         ; get row number
     mov     b, a            ; save in b register
     jnc     GetColumn       ; continue if no error

     lcall print             ; else print error message
     db     "Error - key row not found", 0
     lcall   crlf
KP1:
     lcall   KeyPressed      ; wait until key is released
     jc      KP1
     sjmp    start           ; repeat

GetColumn:
     lcall   FindColumn      ; get column number
     swap    a               ; swap nibbles
     orl     a, b            ; save as b register high nibble
     mov     b, a
     jnc     ReportKey       ; continue if no error

     lcall   print           ; else print error message
     db     "Error - key row not found", 0
     lcall   crlf
KP2:
     lcall   KeyPressed      ; wait until key is released
     jc      KP2
     sjmp    start           ; repeat

ReportKey:
;     lcall   DisplayKey     ; commented out

; ---  show key in hexadecimal on the 7-segment display -----------
     lcall   ConvertKey
     lcall   Display
     mov     P4, a

KP3:
     lcall   KeyPressed      ; wait until key is released
     jc      KP3
     ljmp    start           ; repeat


;====================================================================
; subroutine KeyPressed
; sets the carry flag if a key is pressed
;
; input:   Port 1 bits are connected to a 4 by 4 matrix-type keypad
;          Bits P1.0 through P1.3 are connected to the rows, and
;          P1.4 through P1.7 are connected to the columns.
;
; output:  the carry flag is set if a key is pressed, else the carry
;          flag is cleared.
;
; calls:   nothing
; destroys: a, C
;
;====================================================================
KeyPressed:
     mov     P1, #0Fh          ; set columns to 0, rows to 1
     mov     a, P1             ; read rows
     orl     a, #0F0h          ; mask high nibble
     cpl     a                 ; a is 0 if no key is pressed
     jz      NoKey
     setb    C
     ret
NoKey:
     clr     C
     ret

;====================================================================
; subroutine FindRow
; returns the row number [0-3] of the key pressed in the accumulator
;
; input:   Port 1 bits are connected to a 4 by 4 matrix-type keypad
;          Bits P1.0 through P1.3 are connected to the rows, and
;          P1.4 through P1.7 are connected to the columns.
;
; output:  the row number of the pressed key is returned in a.
;          the carry flag is set if an error occurs, else the carry
;          flag is cleared.
;
; calls:   nothing
; destroys: a, r0, C
;
;====================================================================
FindRow:
     mov     a, P1           ; read rows
     mov     r0, #4          ; initialize counter (R0)

TryNextRow:
     rrc     a               ; read next row
     jnc     RowFound
     djnz    r0, TryNextRow

; --- if loop FindRow does not find the row an error has occured -
     setb    C               ; signal an error
     ret                     ; done

RowFound:
     mov     a, #4           ; number of rows
     subb    a, r0           ; compute row number - note C=0
     clr     C               ; no error
     ret

;====================================================================
; subroutine FindColumn
; returns the column number [0-3] of the key pressed in the accumulator
;
; input:   Port 1 bits are connected to a 4 by 4 matrix-type keypad
;          Bits P1.0 through P1.3 are connected to the rows, and
;          P1.4 through P1.7 are connected to the columns.
;
; output:  the column number of the pressed key is returned in a.
;          the carry flag is set if an error occurs, else the carry
;          flag is cleared.
;
; calls:   nothing
; destroys: a, r0, C
;
;====================================================================
FindColumn:

     mov     r0, #4            ; initialize counter (R0)
     mov     a, #0EFh          ; initialize accumulator ...
     push    acc               ; ... and save on stack

TryNextColumn:
     pop     acc
     mov     P1, a
     rl      a
     push    acc
     mov     a, P1
     orl     a, #0F0h
     cpl     a
     jnz     ColumnFound
     djnz    r0, TryNextColumn
     pop     acc               ; restore stack
     setb    C                 ; signal an error
     ret

ColumnFound:
     pop     acc               ; restore stack
     mov     a, #4             ; number of rows
     clr     C                 ; get ready for the SUBB instruction
     subb    a, r0             ; compute column number
     clr     C                 ; no error
     ret


;====================================================================
; subroutine DisplayKey
; sends a message to the host reporting the row and column number of
; the key pressed
;
; input:   b register holds the row column and row numbers in its
;          nibbles in BCD.
;
; output:  a message is sent to the host
;
; calls:   prthex, crlf
; destroys: a, r0, r2
;
;====================================================================
DisplayKey:
     lcall   print
     db     "Key Pressed - column and row: ",0
     mov     a, b
     lcall   prthex
     lcall   crlf
     ret

;====================================================================
; subroutine ConvertKey
; converts the key row and column of a 4 by 4 matrix-type keypad
; to a hexadecimal number
;
; input:   b register holds the row column and row numbers in its
;          nibbles in BCD.
;
; output:  the hexadecimal number [0-F] is returned in the accumulator.
;
; calls:   nothing
; destroys: a
;
;====================================================================
ConvertKey:
     mov   a, b            ; recall column and row
     rl    a               ; make the row number the more significant
     rl    a               ; two bits
     mov   C, acc.6        ; copy column number low bit to ...
     mov   acc.0, C        ; ... acc.0
     mov   C, acc.7        ; copy column number high bit to ...
     mov   acc.1, C        ; ... acc.1
     anl   acc, #0Fh       ; mask off accumulator high nibble
     ret

;====================================================================
; end of program

