; ----------------------------------------------------
; Math.inc      Last Update:10/24/92
;
; Math routines Example
;
; ====================================================


;===============================================================
; subroutine TC2AV_SB8
; 8-Bit Two's Complement to Absolute Value / Sign Bit Conversion
;
; input:     internal register XT holds a signed byte in two's
;            complement convension
; output:    internal register XA holds the absolute value of the
;            signed byte, addressable bit XS holds the sign (XS is
;            set if XT is a negative number)
; destroys:  a
;================================================================
TC2AV_SB8:

     mov     a, XT          ; read X into accumulator
     jb     acc.7, Xneg     ; X is negative if bit 7 is 1

     mov     XA, XT         ; else, XT is the absolute value
     clr     XS             ; clear sign bit for 'positive'
     ret                    ; done
Xneg:
     cpl     a              ; X is negative so find absolute value
     inc     a              ; XA = copmlement(XT)+1
     mov     XA, a          ; save absolute value
     setb    XS             ; set sign bit for 'negative'
     ret


;====================================================================
; subroutine AV_SB2TC8
; 8-Bit Absolute Value / Sign Bitto Two's Complement Conversion
;
; input:     internal register XA holds the absolute value of the
;          signed byte, addressable bit XS holds the sign (XS is
;          set if XT is a negative number)
; output:     internal register XT holds a signed byte in two's
;          complement convension.  the addressable bit XOV is
;          set if the byte is out of range (-128 to 127).
; destroys:     a
;====================================================================
AV_SB2TC8:
     mov     a, XA               ; get absolute value
     jnb     acc.7, range_OK     ; see if MSB is 0
     cjne     a, #80h, range_error
     jb     XS, range_OK     ; also error if X is positive

range_error:
     setb     XOV          ; set overflow bit
     ret

range_OK:
     clr     XOV          ; no overflow error
     jb     XS, Xneg1     ; check sign
     mov     XT, XA     ; if positive, XT is the same as XA
     ret               ; done
Xneg1:
     mov     a, XA          ; if X is negative, get its absolute value
     cpl     a          ; complement absolute value of X
     inc     a          ; XT = copmlement(XA)+1
     mov     XT, a          ; save two's complement representation
     ret


;===============================================================
; subroutine TC2AV_SB16
; 16-Bit Two's Complement to Absolute Value / Sign Bit Conversion
;
; input:     internal registers XTH and XTL hold a signed word in
;          two's complement convension
; output:     internal registers XAH and XAL hold the absolute
;          value of the signed word, addressable bit XS holds
;          the sign (XS is set if XT is a negative number)
; destroys:     a
;================================================================
TC2AV_SB16:
     mov     a, XTH       ; read X high byte into accumulator
     jb      acc.7, Xneg2 ; X is negative if bit 7 is 1

     mov     XAH, XTH     ; else, XT is the absolute value
     mov     XAL, XTL
     clr     XS           ; clear sign bit for 'positive'
     ret                  ; done
Xneg2:
     setb    XS
     mov     a, XTL       ; X is negative
     cpl     a            ; find its absolute value
     inc     a            ; XA = copmlement(XT)+1
     mov     XAL,a        ; XAL is found
     jnz     XL_OK        ; if not 0, high byte is not incremented

     mov     a, XTH       ; else get high byte
     cpl     a            ; complement high byte...
     inc     a            ; ... and increment
     mov     XAH,a        ; store in XAH
     ret                  ; done
XL_OK:
     mov     a, XTH       ; get high byte
     cpl     a            ; complement high byte - do not increment
     mov     XAH,a        ; store in XAH
     ret                  ; done

;====================================================================
; subroutine AV_SB2TC16
; 16-Bit Absolute Value / Sign Bit to Two's Complement Conversion
;
; input:     internal registers XAH and XAL hold the absolute
;          value of the signed word, addressable bit XS holds
;          the sign (XS is set if XT is a negative number)
; output:     internal registers XTH and XTL hold a signed word in
;          two's complement convension.  the addressable bit XOV is
;          set if the byte is out of range (-32768 to 32767).
; destroys:     a
;====================================================================
AV_SB2TC16:
     mov     a, XAH          ; get absolute value
     jnb     acc.7, range_OK1     ; see if MSB is 0
     clr     acc.7               ; see if X=-8000h
     orl     a, XAL          ; acc=0 only if X=8000
     jnz     range_error1     ; else error
     jb     XS, range_OK1     ; also error if X is positive

range_error1:
     setb     XOV          ; set overflow bit
     ret

range_OK1:
     clr     XOV          ; no overflow error
     jb     XS, Xneg2     ; check sign
     mov     XTH, XAH     ; if positive, XT is the same as XA
     mov     XTL, XAL
     ret               ; done
Xneg3:
     mov     a, XAL     ; if X is negative, get its absolute value
     cpl     a          ; complement
     inc     a          ; XT = copmlement(XA)+1
     mov     XTL, a
     jnz     XL_OK1     ; if not 0, high byte is not incremented

     mov     a, XAH     ; else get high byte
     cpl     a          ; complement high byte...
     inc     a          ; ... and increment
     mov     XTH,a      ; store in XTH
     ret               ; done
XL_OK1:
     mov     a, XAH     ; get high byte
     cpl     a          ; complement high byte - do not increment
     mov     XTH,a      ; store in XTH
     ret               ; done

;===============================================================
; subroutine SMUL8
; 8-Bit Multiplication of Signed Integers in the Absolute Value
; / Sign Bit Format
;
; input:     internal registers XA and YA hold the absolute
;          values of the signed bytes X and Y.  addressable
;          bits XS and YS hold the sign bits, set if the
;          corresponding numbers are negative
; output:     internal registers ZAH and ZAL hold the absolute
;          value of the result Z=XY.  addressable bit ZS
;          holds the sign of the result, set if the result is
;          negative.
; destroys:     a
;================================================================
SMUL8:
     mov     a, XA          ; read X and ...
     mov     b, YA          ; ... Y
     mul     ab             ; multiply X and Y
     mov     ZH, b          ; save result high ...
     mov     ZL, a          ; ... and low byte

     jb      XS, SM8_2      ; get sign of X
     jb      YS, SM8_1
     clr     ZS
     ret
SM8_1:
     setb    ZS
     ret
SM8_2:
     jb      YS, SM8_3
     setb    ZS
     ret
SM8_3:
     clr     ZS
     ret

;===============================================================
; subroutine SDIV8
; 8-Bit Division of Signed Integers in the Absolute Value /
; Sign Bit Format
;
; input:     internal registers XA and YA hold the absolute
;          values of the signed bytes X and Y.  addressable
;          bits XS and YS hold the sign bits, set if the
;          corresponding numbers are negative
; output:     internal registers ZAQ and ZAR hold the quotient and
;          the remainder of the division Z=XY.  addressable
;          bit ZS holds the sign of the result, set if the
;          result is negative.  the addressible bit ZOV is set
;          if Y is zero.
; destroys:     a
;================================================================
SDIV8:
     mov     a, XA          ; read X and ...
     mov     b, YA          ; ... Y
     div     ab          ; divide X and Y
     mov     C, OV          ; get overflow flag
     mov     ZOV, C     ; save in ZOV.  (set if Y=0)
     mov     ZAQ, a     ; save result quotient
     mov     ZAR, b     ; save remainder

     jb      XS, SD8_2      ; get sign of X
     jb      YS, SD8_1
     clr     ZS
     ret
SD8_1:
     setb    ZS
     ret
SD8_2:
     jb      YS, SD8_3
     setb    ZS
     ret
SD8_3:
     clr     ZS
     ret

;====================================================================
; subroutine ADD16
; 16-Bit Unsigned Addition
;
; input:     internal registers XH, XL, YH, and YL hold the high and
;          low bytes of the unsigned words
; output:     internal registers ZH and ZL hold high and low bytes of
;          the unsigned word Z=X+Y.  addressible bit ZOV is set if
;          the result (Z) is out of range (an external carry is
;          produced from the high byte)
; destroys:     a, flags
;====================================================================
ADD16:
     mov     a, XL          ; load X low byte into accumulator
     add     a, YL          ; add Y low byte
     mov     ZL, a          ; put result in Z low byte
     mov     a, XH          ; load X high byte into accumulator
     addc     a, YH          ; add Y high byte with the carry from low ...
                    ; ... byte operation
      mov     ZH, a          ; save result in Z high byte
     mov     ZOV, C     ; set ZOV if external carry
     ret               ; done


;====================================================================
; subroutine SUB16
; 16-Bit Unsigned subtraction
;
; input:     internal registers XH, XL, YH, and YL hold the high and
;          low bytes of the unsigned words
; output:     internal registers ZH and ZL hold high and low bytes of
;          the signed word Z=X-Y.  addressible bit ZOV is set if
;          the result (Z) is out of range (if an external barrow is
;          produced)
; destroys:     a, flags
;====================================================================
SUB16:
     mov     a, XL          ; load X low byte into accumulator
     clr     C          ; clear carry flag
     subb     a, YL          ; subract Y low byte
     mov     ZL, a          ; put result in Z low byte
     mov     a, XH          ; load X high byte into accumulator
     subb     a, YH          ; subtract Y high byte with the barrow ...
                    ; ... from low byte operation
     mov     ZH, a          ; save result in Z high byte
     mov     ZOV, C     ; set ZOV if an external barrow is produced
     ret               ; done
;====================================================================
; subroutine SUB32
; 32-Bit Unsigned subtraction
;
; input:     internal registers X3, X2, X1, X0, and Y3, Y2, Y1, and Y0
;          hold the 32-bit unsigned integers
; output:     internal registers Z3, Z2, Z1, and Z0 hold the result
;          Z=X-Y.  addressible bit ZOV is set if the result (Z) is
;          out of range (if an external barrow is produced)
; destroys:     a, flags
;====================================================================
SUB32:
     mov     a, X0          ; load X low byte into accumulator
     clr     C              ; clear carry flag
     subb    a, Y0          ; subract Y low byte
     mov     Z0, a          ; put result in Z low byte
     mov     a, X1          ; repeat with other bytes...
     subb    a, Y1
     mov     Z1, a
     mov     a, X2
     subb    a, Y2
     mov     Z2, a
     mov     a, X3
     subb    a, Y3
     mov     Z3, a
     mov     ZOV, C         ; set ZOV if an external barrow is produced
     ret                    ; done
;====================================================================
; subroutine BMW
; Byte Multiplied by 16-Bit Word
;
; input:     internal register X holds the 8-bit unsigned integer.
;          internal registers YH, and YL hold the high and
;          low bytes of the unsigned word.
; output:     internal registers Z2, Z1 and Z0 hold the three result
;          bytes.  Z2 is the most significant byte, and Z0, the least
;          significant.  The result always fits in 3 bytes.
; destroys:     a, b, r0, flags
;====================================================================
BMW:
     mov     psw, #0     ; select register bank 0
     mov     a, X        ; load X into accumulator
     mov     b, YL       ; load Y low byte into b register
     mul     ab          ; multiply
     mov     Z0, a       ; save result low byte
     push     b          ; push result high byte
                         ; this byte needs to be added to the low
                         ; byte of the product X*YH

     mov     a, X        ; load X into accumulator
     mov     b, YH       ; load Y high byte into b register
     mul     ab          ; multiply
     pop     0           ; pop r0 to recall X*YL high byte
     add     a, r0       ; add X*YL high byte and X*YH low byte
     mov     Z1, a       ; save result
     clr     a           ; clear accumulator
     addc     a, b       ; a = b + carry flag
     mov     Z2, a       ; save result
     ret                 ; done
;====================================================================
; subroutine MUL16
; Multiply two 16-Bit Unsigned Words
;
; input:     internal register XH, XL, YH, and YL hold the high and
;          low bytes of the two unsigned words X and Y.
; output:     internal registers Z3, Z2, Z1 and Z0 hold the four result
;          bytes.  Z3 is the most significant byte, and Z0, the least
;          significant.  The result always fits in 4 bytes.
; destroys:     a, b, r0, r1, flags
;====================================================================
MUL16:
     lcall BMW          ; multiply XL with (YH:YL)
     mov     a, Z2
     push    acc        ; push ZL2
     mov     a, Z1
     push    acc        ; push ZL1
     mov     a, Z0
     push    acc        ; push ZL0

     mov     XL, XH     ; put XH in XL
     lcall   BMW        ; multiply XH with (YH:YL)

     mov     b, Z0      ; save ZH0 in b register
     pop     Z0         ; recall ZL0
     pop     acc        ; restore ZL1 in accumulator
     add     a, b       ; add ZH0 and ZL1
     mov     b, Z1      ; save Z1 in b register
     mov     Z1, a      ; save result
     pop     acc        ; restore ZL2 in accumulator
     addc    a, b       ; add ZH1 and ZL2 with carry from ZH0+ZL1
     mov     b, Z2      ; save ZH2 in b register
     mov     Z2, a      ; save ZH1+ZL2+C in Z2
     clr     a          ; clear accumulator
     addc    a, b       ; add ZH2 with carry from ZH1+ZL2
     mov     Z3, a      ; save result
     ret                ; done

     mov     a, X       ; load X into accumulator
     mov     b, YL      ; load Y low byte into b register
     mul     ab         ; multiply
     mov     Z0, a      ; save result low byte
     push    b          ; push result high byte
                        ; this byte needs to be added to the low
                        ; byte of the product X*YH

     mov     a, X       ; load X into accumulator
     mov     b, YH      ; load Y high byte into b register
     mul     ab         ; multiply
     pop     0          ; pop r0 to recall X*YL high byte
     add     a, r0      ; add X*YL high byte and X*YH low byte
     mov     Z1, a      ; save result
     clr     a          ; clear accumulator
     addc    a, b       ; a = b + carry flag
     mov     Z2, a      ; save result
     ret                ; done

;====================================================================
; subroutine DIV16
; 16-Bit Unsigned Division
;
; input:     internal registers XH, XL, YH, and YL hold the high and
;          low bytes of the unsigned dividend and divisor.
; output:     internal registers Z3 and Z2 hold high and low bytes of
;          the quotient of Z=XY.  internal registers Z1 and Z0 hold
;          the high and low bytes of the remainder.  addressible bit
;          ZOV is set if Y=0, i.e., the result is out of range.
;
;          r1 and r0 store the high and low bytes of the partial
;          remainder.  r3 and r2 hold the partially computed quotient.
;
; calls:     SUB16
; destroys:     a, r0, r1, r2, r3, r7, flags
;====================================================================
DIV16:
     mov     a, YH           ; get divisor high byte
     orl     a, YL           ; OR with low byte
     jnz     div_OK          ; divisor OK if not 0
     setb     ZOV            ; else, overflow
     ret
div_OK:
     mov     r1, XH          ; store dividend in r1, r0
     mov     r0, XL
     mov     XH, #0          ; clear partial remainder
     mov     XL, #0
     mov     r3, #0          ; clear partial quotient
     mov     r2, #0
     mov     r7, #16         ; set loop count

div_loop:
     clr     C               ; clear carry flag
     mov     a, r0           ; shift the highest bit of the dividend...
     rlc     a               ; ... into...
     mov     r0, a
     mov     a, r1
     rlc     a
     mov     r1, a

     mov     a, XL           ; ... the lowest bit of the partial ...
     rlc     a               ; ... remainder
     mov     XL, a
     mov     a, XH
     rlc     a
     mov     XH, a

     lcall   SUB16           ; attempt to subtract to see if the...
                             ; ... partial remainder is as large or...
                             ; ... larger than the divisor.

     mov     C, ZOV          ; get subtraction external barrow
     cpl     C               ; complement external barrow
     jnc     div_1           ; do not update partial remainder if no barrow

     mov     XH, ZH          ; update partial quotient
     mov     XL, ZL
div_1:
     mov     a, r2           ; add result bit to partial quotient
     rlc     a
     mov     r2, a
     mov     a, r3
     rlc     a
     mov     r3, a
     djnz     r7, div_loop

     mov     Z3, r3          ; put quotient in Z3, and Z2
     mov     Z2, r2
     mov     Z1, XH          ; get remainder, saved before the...
     mov     Z0, XL          ; last subtraction
     clr     ZOV             ; divisor is not 0
     ret                     ; done


;====================================================================
; subroutine DIV32
; 32-Bit Unsigned Division
;
; input:     internal registers X3, X2, X1, and X0 hold the 32-bit
;          unsigned dividend.  similarly, internal registers Y1 and
;          Y0 hold the high and low bytes of the unsigned divisor.
;
; output:     internal registers Z5, Z4, Z3, and Z2 hold the 32-bit
;          quotient of Z=XY.  internal registers Z1 and Z0 hold
;          the high and low bytes of the remainder.  addressible bit
;          ZOV is set if Y=0, i.e., the result is out of range.
;
;          internal registers PR3, PR2, PR1, and PR0 store the
;          partial remainder.  r5, r4, r3, and r2 hold the partially
;          computed quotient.
;
; calls:     SUB16
; destroys:     a, r2, r3, r4, r5, r7, flags
;====================================================================
DIV32:
     mov     a, Y1           ; get divisor high byte
     orl     a, Y0           ; OR with low byte
     jnz     div32_OK        ; divisor OK if not 0
     setb    ZOV             ; else, overflow
     ret
div32_OK:
     mov     Y2, #0          ; high 16 bits of Y is 0
     mov     Y3, #0
     mov     PR3, X3         ; store dividend
     mov     PR2, X2
     mov     PR1, X1
     mov     PR0, X0
     mov     X3, #0          ; clear partial remainder
     mov     X2, #0
     mov     X1, #0
     mov     X0, #0
     mov     r6, #0
     mov     r5, #0
     mov     r4, #0          ; clear partial quotient
     mov     r3, #0
     mov     r7, #32         ; set loop count

div32_loop:

     clr     C               ; clear carry flag
     mov     a, PR0          ; shift the highest bit of the dividend...
     rlc     a               ; ... into...
     mov     PR0, a
     mov     a, PR1
     rlc     a
     mov     PR1, a
     mov     a, PR2
     rlc     a
     mov     PR2, a
     mov     a, PR3
     rlc     a
     mov     PR3, a

     mov     a, X0           ; ... the lowest bit of the partial ...
     rlc     a               ; ... remainder
     mov     X0, a
     mov     a, X1
     rlc     a
     mov     X1, a
     mov     a, X2
     rlc     a
     mov     X2, a
     mov     a, X3
     rlc     a
     mov     X3, a

     lcall   SUB32           ; attempt to subtract to see if the...
                             ; ... partial remainder is as large or...
                             ; ... larger than the divisor.

     mov     C, ZOV          ; get subtraction external barrow
     cpl     C               ; complement external barrow
     jnc     div32_1         ; do not update partial remainder if no barrow

     mov     X3, Z3          ; update partial quotient
     mov     X2, Z2
     mov     X1, Z1
     mov     X0, Z0
div32_1:
     mov     a, r3           ; add result bit to partial quotient
     rlc     a
     mov     r3, a

     mov     a, r4
     rlc     a
     mov     r4, a

     mov     a, r5
     rlc     a
     mov     r5, a

     mov     a, r6
     rlc     a
     mov     r6, a

     djnz     r7, div32_loop

     mov     Z5, r6          ; put quotient in Z3, and Z2
     mov     Z4, r5
     mov     Z3, r4
     mov     Z2, r3
     mov     Z1, X1          ; get remainder, saved before the...
     mov     Z0, X0          ; last subtraction
     clr     ZOV             ; divisor is not 0
     ret                     ; done
;====================================================================
; subroutine SADD16
; 16-Bit Signed (Two's Complement) addition
;
; input:     internal registers XH, XL, YH, and YL hold the high and
;          low bytes of the signed words in two's complement
;          convension
; output:     internal registers ZH and ZL hold high and low bytes of
;          the signed word Z=X+Y.  addressible bit ZOV is set if
;          the result (Z) is out of range (-32768 to 32767)
; destroys:     a, flags
;====================================================================
SADD16:
     mov     a, XL          ; load X low byte into accumulator
     add     a, YL          ; add Y low byte
     mov     ZL, a          ; put result in Z low byte
     mov     a, XH          ; load X high byte into accumulator
     addc    a, YH          ; add Y high byte with the carry from low ...
                            ; ... byte operation
     mov     ZH, a          ; save result in Z high byte
     mov     C, OV          ; get the overflow flag and ...
     mov     ZOV, C         ; ... transfer to bit ZOV
     ret                    ; done


;====================================================================
; subroutine SSUB16
; 16-Bit Signed (Two's Complement) subtraction
;
; input:     internal registers XH, XL, YH, and YL hold the high and
;          low bytes of the signed words in two's complement
;          convension
; output:     internal registers ZH and ZL hold high and low bytes of
;          the signed word Z=X-Y.  addressible bit ZOV is set if
;          the result (Z) is out of range (-32768 to 32767)
; destroys:     a, flags
;====================================================================
SSUB16:
     mov     a, XL          ; load X low byte into accumulator
     clr     C              ; clear carry flag
     subb     a, YL         ; subract Y low byte
     mov     ZL, a          ; put result in Z low byte
     mov     a, XH          ; load X high byte into accumulator
     subb     a, YH         ; subtract Y high byte with the barrow ...
                            ; ... from low byte operation
     mov     ZH, a          ; save result in Z high byte
     mov     C, OV          ; get the overflow flag and ...
     mov     ZOV, C         ; ... transfer to bit ZOV
     ret                    ; done

;===============================================================
; subroutine SMUL16
; multiplication of signed 16-Bit words in the Absolute Value /
; Sign Bit format
;
; input:     internal registers XH, XL and YH, YL hold the
;          magnitudes of the two signed words to be multiplied.
;          The addressible bits XS and YS are set if the
;          corresponding number is negative.
; output:     the absolute value of the result is placed in the 4
;          internal registers named Z3, Z2, Z1, and Z0.
;          Addressable bit ZS is set if the result is negative.
; calls:     MUL16
; destroys:     a
;================================================================
SMUL16:
     lcall     MUL16          ; multiply absolute values
     mov     C, XS          ; get sign of X

     jb      XS, SM16_2      ; get sign of X
     jb      YS, SM16_1
     clr     ZS
     ret
SM16_1:
     setb    ZS
     ret
SM16_2:
     jb      YS, SM16_3
     setb    ZS
     ret
SM16_3:
     clr     ZS
     ret

;===============================================================
; subroutine SDIV16
; division of signed 16-Bit words in the Absolute Value / Sign
; Bit format
;
; input:     internal registers XAH, XAL and YAH, YAL hold the
;          magnitudes of the two signed words.  the addressable
;          bits XS and YS are set if the corresponding number is
;          negative.
; output:     the absolute value of the result Z=XY is placed in
;          the 4 internal registers named Z3, Z2, Z1, and Z0.
;          addressable bit ZS is set if the result is negative,
;          and ZOV is set if Y=0.
; calls:     DIV16
; destroys:     a
;================================================================
SDIV16:
     lcall     DIV16         ; divide absolute values
     jb      XS, SD16_2      ; get sign of X
     jb      YS, SD16_1
     clr     ZS
     ret
SD16_1:
     setb    ZS
     ret
SD16_2:
     jb      YS, SD16_3
     setb    ZS
     ret
SD16_3:
     clr     ZS
     ret


;====================================================================
; subroutine FMUL
; multiply two floating point numbers
;
; input:     internal registers X2, X1, X0 and Y2, Y1, Y0 hold the
;          two floating point numbers to be multiplied.
; output:     the product Z = X  Y is saved in Z2, Z1, Z0.
; calls:     MUL16
; destroys:     a, b, r2, r3, r4
;====================================================================
FMUL:
     mov     a, X1          ; get the sign bit plus 7 mantissa bits of X
     mov     C, acc.7       ; read sign bit
     mov     XS, C          ; save sign bit
     mov     a, Y1          ; get the sign bit plus 7 mantissa bits of Y
     mov     C, acc.7       ; read sign bit
     mov     YS, C          ; save sign bit

     jb      XS, FMUL_2      ; get sign of X
     jb      YS, FMUL_1
     clr     ZS
     sjmp    FMUL_4
FMUL_1:
     setb    ZS
     sjmp    FMUL_4
FMUL_2:
     jb      YS, FMUL_3
     setb    ZS
     sjmp    FMUL_4
FMUL_3:
     clr     ZS

FMUL_4:
     mov     r2, X2         ; save exponent of X in r2
     mov     r3, Y2         ; save exponent of Y in r3

     cjne    r2, #0, XNZ    ; see if X is zero
     mov     Z2, #0         ; if X=0, then Z=0
     mov     Z1, #0
     mov     Z0, #0
     ret                    ; done
XNZ:
     cjne    r3, #0, YNZ    ; see if Y is zero
     mov     Z2, #0         ; if Y=0, then Z=0
     mov     Z1, #0
     mov     Z0, #0
     ret                    ; done
YNZ:
     mov     a, r2          ; get exponent of X
     add     a, r3          ; add exponent of Y
     mov     b, #0          ; clear b register
     mov     b.0, C         ; move carry bit into b register
                            ; b register is high byte of sum of exponents
     clr     C
     subb    a, #128        ; subtract 128 from sum of exponents low byte
     mov     r2, a          ; save exponent in r2
     mov     a, b           ; get sum of exponents high byte
     subb    a, #0          ; subtract high bytes
     jnc     NoUNF          ; external barrow means underflow
     mov     Z2, #0         ; underflow...set Z=0
     mov     Z1, #0         ; it may be more appropriate in some
     mov     Z0, #0         ; applications to set ZOV and signal en error
     ret                    ; done
NoUNF:
     jz      NoOVF          ; if high byte is 1 then overflow
     setb    ZOV            ; set overflow error flag
     ret                    ; done

; ---     if the program reaches NoOVF, the exponent is in r2 ---
; --- and the sign is in ZS.  next multiply mantissas     ---
NoOVF:
     mov     a, X1          ; get X1
     setb     acc.7          ; set the implied MSB=1
     mov     X1, a          ; save completed mantissa of X
     mov     a, Y1          ; get Y1
     setb     acc.7          ; set the implied MSB=1
     mov     Y1, a          ; save completed mantissa of Y

     lcall MUL16          ; multiply the mantissas

     mov    a, Z3         ; get mantissa high byte
     jb     acc.7, XYOK   ; mantissa is OK if MSB is 1

; --- if MSB of XY is 0, then the mantssa is shifted left ---
; --- and the exponent is incremented -----------------------
     inc    r2            ; this may result in an overflow
     cjne   r2, #0, EXP_OK
     setb   ZOV           ; overflow
     ret

EXP_OK:
     clr    C
     mov    a, Z1
     rlc    a
     mov    a, Z2
     rlc    a
     mov    Z0, a
     mov    a, Z3
     rlc    a

     mov    C, ZS
     mov    acc.7, C      ; sign bit in place of the implied '1'
     mov    Z1, a
     mov    Z2, r2
     ret
XYOK:
     mov    Z0, Z2        ; mantissa low byte
     mov    a, Z3         ; mantissa high byte
     mov    C, ZS
     mov    acc.7, C      ; sign bit in place of the implied '1'
     mov    Z1, a
     mov    Z2, r2       ; get exponent
     ret                  ; done

; ====================================================
;
; end of include module