; ----------------------------------------------------
; PULSE.ASM          Last Update:10/27/92
;
;  Pulse and Period Measurement using Timer 2 Capture
;  Function
;
;  This program uses the SAB80515 Timer 2 capture
;  registers to measure the duration of a positive
;  going pulse.  The timer registers T2H and T2L may
;  be  copied to the Compare/Capture Registers CRCH
;  and CRCL upon an external signal.  The period is
;  assumed to be shorter that 2^16 machine cycles.
;
;  Two bits, SignalHigh and Trigger, are used as flags.
;  The flag Trigger is set by the main loop to allow
;  the capture of CRC registers in the EX3 Interrupt
;  Service Routine(ISR).  The ISR clears Trigger when
;  the pulse end is detected and its count stored.
;
;  For Pulse measurements, SignalHigh is set when a
;  0-to-1 transition is detected.  It is used to
;  determine whether to store the next count as the
;  begin or end count. That is, if SignalHigh is 0,
;  the next capture is stored in BeginL and BeginH.
;  Otherwise, the next capture is stored in EndL and
;  EndH.  The portion of code in the ISR for external
;  interrupt 3 when SignalHigh is set contains an
;  instruction to clear the bit I3FR of the Timer 2
;  Control Register T2CON.  This bit determines whether
;  the interrupt is generated on a 0-to-1 transition or
;  a 1-to-0 transition.  When I3FR is cleared, a 1-to-0
;  transition couses an interrupt.  Not clearing this
;  bit would cause the next interrupt to be generated
;  also at a 0-to-1 transition.  Thus, the time between
;  the two interrupts is the time between two 0-to-1
;  transitions.  The difference is the period of the
;  signal.  If I3FR is cleared after the first
;  transition, the next interrupt occurs at the 1-to-0
;  transition.  Thus the difference in counts is the width
;  of the positive going pules.  In order to measure the
;  width of a negative pulse, I3FR should be cleared at
;  the start and set in the ISR for external interrupt 3.
;
;  The program asks for a key input from the host and
;  continues until a Control-C is received.
;
; ====================================================
; UIOD connections
;
;  Connect P1.0 to the input TTL-level pulse.
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; constants
; internal registers:
  BeginL     equ  7Ch   ; beginning count low  byte
  BeginH     equ  7Dh   ; beginning count high byte
  EndL       equ  7Eh   ; end count low  byte
  EndH       equ  7Fh   ; end count high byte
; bits
  SignalHigh equ 10h    ; set at 0-to-1 transition
                        ; and cleared at 1-to-0
  Trigger    equ 11h    ; set to start captures
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 crlf      equ  0115h  ; a
 getchr    equ  0121h  ; a
 os_return equ  0130h
 print     equ  0136h  ; a, dptr
 prsphx    equ  0139h  ; a, r2
 setintvec equ  0145h  ; a, dptr
; ----------------------------------------------------
; --- port definitions -------------------------------
; output ports
  PulseIn  equ   90h   ; port bit P1.0
; ----------------------------------------------------
; --- origin at the beginning of RAM -----------------
   org   8000h
; --- program starts here ----------------------------

    lcall print
    db "begin initialization...", 10,13,0

    mov   T2CON, #01010000b ; initialize Timer 2
                            ;  T2I1=0 and T2I0=0
                            ;  T2CM=0
                            ;  T2R1=1 and T2R0=0
                            ;  I3FR=1 (capture upon 0-to-1)
                            ;  I2FR=0
                            ;  T2PS=0
    mov   CCEN, #01         ; CRC capture upon external signal

    mov   a, #5             ; T2 interrupt is the fifth source
    mov   dptr, #T2Isr      ; interrupt service routine for T2
    lcall setintvec         ; set interrupt vector

    mov   a, #8             ; EX3 interrupt is the eigth source
    mov   dptr, #EX3Isr     ; interrupt service routine for EX3
    lcall setintvec         ; set interrupt vector

    setb  ET2               ; allow T2 interrupts
    setb  EX3               ; allow EX3 interrupts
    clr   Trigger


    lcall print
    db "...initialization done", 10,13,0

loop:                       ; each iteration makes a pulse width measurement
    clr   SignalHigh        ; initialize pulse flag
    mov   CRCL, #0
    mov   CRCH, #0
    setb  EAL               ; allow interrupts
    setb  T2I0              ; start Timer 2
    setb  Trigger           ; trigger measurement
here:
    jb    Trigger, Here     ; wait until pulse is measured
    clr   T2I0              ; stop Timer 2
    clr   EAL               ; disable all interrupts
    clr   TF2               ; clear timer 2 overflow flag - just in case...
; --- the pulse width measurement is complete.  The end and begin -----
; --- counts are stored in registers EndL, EndH, BeginL, and BeginH ---
; --- Note that the timer and the interrupts are disabled -------------

    clr   C                 ; compute pulse width
    mov   a, EndL
    subb  a, BeginL
    mov   r0, a             ; save pulse width low byte in r0
    mov   a, EndH
    subb  a, BeginH
    mov   r1, a             ; save pulse width low byte in r1

    lcall ShowResults       ; display measurement results on Host


; --- ask host whether to continue or quit ----------------------------
    lcall print
    db   "press any key to continue or Ctl-C to quit...", 0
    lcall crlf
    lcall crlf
    lcall getchr
    cjne  a, #3, Again
    lcall print             ; print message
    db    10,13, "CTL-C received - program terminates", 10,13,0
    clr   ET2               ; disable T2 interrupts
    clr   EX3               ; disable EX3 interrupts
    ljmp  os_return         ; return to OS

Again:
    ljmp loop
; ====================================================
;  subroutine T2Isr
;
;  TF2 must be cleared by software
;
;  input    : Timer 2 overflow
;  output   : nothing
;  destroys : nothing
; ----------------------------------------------------
T2Isr:
     clr   TF2           ; clear overflow flag
     reti


; ====================================================
;  subroutine Ex3Isr
;
;  External Interrupt 3
;
;  input    : External Interrupt 3
;  output   : updates registers BeginL, BeginH, EndL,
;             EndH, and flag SignalHigh
;  destroys : nothing
; ----------------------------------------------------
EX3Isr:
     jnb   Trigger, EX3done
     jb    SignalHigh, SH
     mov   BeginL, CRCL
     mov   BeginH, CRCH
     mov   CRCL, #0
     mov   CRCH, #0
     setb  SignalHigh
; --- remove (or comment out) the following ----------
; --- instruction to measure the period --------------
     clr   I3FR            ; next capture upon 1-to-0
     reti
SH:
     mov   EndL, CRCL
     mov   EndH, CRCH
     mov   CRCL, #0
     mov   CRCH, #0
     clr   SignalHigh
     clr   Trigger
     setb  I3FR
EX3done:
     reti

; ====================================================
;  subroutine ShowResults
;
;  Displays pulse measurement results
;
;  input    : Pulse Width in R1 and R0; begin count in
;             registers BeginL and BeginH; end count in
;             EndL and EndH.
;  output   : writes messages to host screen
;  destroys : a, r2, dptr
; ----------------------------------------------------
ShowResults:
    lcall print             ; print pulse width
    db   "pulse width : ", 0
    mov   a, r1
    lcall prsphx
    mov   a, r0
    lcall prsphx
    lcall crlf

    lcall print             ; print Timer2 count at pulse end
    db   "end count   : ", 0
    mov   a, EndH
    lcall prsphx
    mov   a, EndL
    lcall prsphx
    lcall crlf

    lcall print             ; print Timer2 count at pulse start
    db   "begin count : ", 0
    mov   a, BeginH
    lcall prsphx
    mov   a, BeginL
    lcall prsphx
    lcall crlf
    lcall crlf
    ret

; ====================================================
;
; end of program
