; ----------------------------------------------------
;  PWMT2.ASM          Last Update:10/27/92
;
;  Pulse Width Modulation usint Timer 2 Compare
;
;  This program uses the SAB80515 Timer 2 compare
;  registers to implement Pulse Width Modulation (PWM).
;  More specifically, the frequency and duty cycle of a
;  square wave output is selected by programming the
;  Programmable Timer/Counter Register Array (PTRA).
;  The CPU is involved in setting up and controlling
;  the PTRA unit.  Otherwise, the PWM signal is
;  generated without any overhead to the CPU.  This
;  makes the PTRA unit a highly efficient and precise
;  PWM signal generator.  The PWM signal may be
;  regulated by a low-pass filter to generate a voltage
;  proportional to the duty cycle.  This scheme is often
;  used with microcontrollers as a digital-to-analog
;  conversion technique.
;
;  Timer 2 of the 80515 microcontroller PTRA is used.
;  Timer 2 is initialized to run in the auto-reload mode.
;  This is a 16-bit timer, incremented every machine cycle.
;  Each overflow reloads the low and high count register
;  CRCL and CRCH into the counter registers TL2 and TH2.
;  If the reload value is 0, with a 12MHz crystal, Timer 2
;  overflows every 65536 microseconds, and if FFFF, it
;  overflows every microsecond.  The Compare/Capture
;  register2 CCL1 and CCH1 hold the compare value.
;  Compare mode 0 is used.  When TL2 and TH2 match CCL1
;  and CCH1, the corresponding output bit P1.1 is set
;  (made 1).  P1.1 is cleared (made 0) upon Timer 2
;  overflow.
;
; ====================================================
; UIOD connections
;
;  Connect P1.1 is the PWM signal output.
;  Connect P1.1 to an LED and observe the brightness
;  of the LED
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; constants
; bytes:
  PeriodL    equ   0    ; period low  byte (reload value)
  PeriodH    equ   0   ; period high byte (reload value)
  OnPeriodL  equ   0    ; on period low byte
  OnPeriodH  equ  80h   ; on period high byte
; ----------------------------------------------------
; system calls              registers used
; ----------------------------------------------------
 ascbin    equ  0100h  ; a, r2, error flag
 beep      equ  0106h  ;  none
 crlf      equ  0115h  ; a
 getchr    equ  0121h  ; a
 inkey     equ  012ah  ; a
 os_return equ  0130h
 print     equ  0136h  ; a, dptr
 prsphx    equ  0139h  ; a, r2
 setintvec equ  0145h  ; a, dptr
 sndchr    equ  0148h  ; a
;bits
 errorf    equ     3   ; monitor error flag
; ----------------------------------------------------
; --- port definitions -------------------------------
; output ports
  PWM equ   90h   ; port bit P1.0
; ----------------------------------------------------
; --- origin at the beginning of RAM -----------------
   org   8000h
; --- program starts here ----------------------------

    lcall print
    db "about to begin initialization...", 10,13,0

    mov   CRCL, #PeriodL    ; initialize reload values...
    mov   CRCH, #PeriodH    ; ... (determines the period)

    mov   CCL1, #OnPeriodL  ; initialize compare values...
    mov   CCH1, #OnPeriodH  ; ... (determines the duty cycle)

    mov   T2CON, #00010000b ; initialize Timer 2
                            ;  T2I1=0 and T2I0=0
                            ;  T2CM=0
                            ;  T2R1=1 and T2R0=0
                            ;  I3FR=0 and I2FR=0
                            ;  T2PS=0
    mov   CCEN, #10101000b  ; initialize Capture/Compare
                            ;    Enable Register

    mov   a, #5             ; T2 interrupt is the fifth source
    mov   dptr, #T2Isr      ; interrupt service routine for T2
    lcall setintvec         ; set interrupt vector

    setb  ET2               ; allow T2 interrupts
    setb  EAL               ; allow interrupts
    setb  T2I0              ; start Timer 2

    lcall print
    db "...initialization done", 10,13,0

loop:                       ; this loop is the foreground task
    lcall crlf              ; new line
    lcall print
    db    "input duty cycle [0-F] : ",0
    lcall getchr
    cjne  a, #3, noCtlC
    lcall print             ; print message
    db    10,13, "CTL-C received - program terminates", 10,13,0
    ljmp  os_return         ; return to OS

noCtlC:
    mov   b, a              ; store character in b register
    clr   errorf            ; reset error flag
    lcall ascbin            ; convert to a (binary) number
    jb    errorf, loop      ; abort if char out of range
    mov   a, b              ; recall character
    lcall sndchr            ; echo character
    mov   a, b              ; recall character
    swap  a                 ; multiply by 16
    mov   CCH1, a
    sjmp  loop


; ====================================================
;  subroutine T2Isr
;
;  TF2 must be cleared by software
;
;  input    : Timer 2 overflow
;  output   : nothing
;  destroys : nothing
; ----------------------------------------------------
T2Isr:
     clr   TF2           ; clear overflow flag
     reti

; ====================================================
;
; end of program
