; module R-NET.INC used with R-NETx.ASM
;
; Courtesy Rigel Corporation
;
; =============================================================================
; external memory
  InBuffer  equ 8800h ; serial  input buffer

; =============================================================================
; subroutine Sout - RS-485 Serial Out
;
; This routine sends one byte out the RS-485 driver.
;
; This is the lowest level routine to send messages.
; All error checks are performed at a higher level.
;
;  input    : acc holds byte to be sent
;  output   : none
;  destroys : R0, carry flag
; -----------------------------------------------------------------------------
Sout:
       push  acc          ; save data byte

       setb  Dir485       ; direction = 1 for output
       clr   Data485      ; send start bit

       lcall BaudDelay    ; delay for start bit

       mov   R0, #8       ; bit count
SO1:
       pop   acc          ; recall data byte
       mov   C, acc.0     ; move LSB via carry flag ...
       mov   Data485, C   ; ... to the RS-485 data port
       rr    a            ; rotate accumulator - set up to send next bit
       push  acc          ; save acc on stack
       lcall BaudDelay    ; delay for 1 bit transmission
       djnz  R0, SO1      ; repeat 8 times, once for each bit

       setb  Data485      ; place stop bit (=1)
       lcall BaudDelay    ; delay for stop bit
       lcall BaudDelay    ; extra delay for stop bit - to be safe
       clr   Dir485       ; direction = 0 for input from RS-485 port

       mov   a, #10
       lcall delay

       pop   acc          ; recall the data byte
       ret

; =============================================================================
; subroutine Sin - RS-485 Serial Input
;
; This routine reads a byte out the RS-485 driver.
; This routine must be called after the start bit is
; detected.
;
; This is the lowest level routine to receive messages.
; All error checks are performed at a higher level.
;
;  input    : none
;  output   : acc returns the byte received
;  destroys : noting
; -----------------------------------------------------------------------------
Sin:
       mov   PSW, #0         ; select bank 0
       push  0               ; save R0
       push  acc             ; partially received byte is kept on stack

       lcall BaudDelay       ; delay for end of start bit
       mov   R0, #8          ; bit count
SI1:
       pop   acc             ; get bits received so far
       mov   C, Data485      ; read next bit from network
       mov   acc.0, C        ; move bit to acc LSB via carry flag
       rr    a               ; rotate to get ready for next bit
       push  acc             ; save bits received so far on stack
       lcall BaudDelay       ; wait for the next bit
       djnz  R0, SI1         ; repeat 8 times, once for each bit

       lcall BaudDelay       ; wait for the stop bit

       clr   Dir485          ; direction = 0 for input from RS-485 port

       pop   acc             ; recall data byte - all bits received
       pop   0               ; restore R0
       ret

; =============================================================================
; subroutine BaudDelay
; Suspends the program for a constant period
; -----------------------------------------------------------------------------
BaudDelay:
       lcall BaudDelayShort   ; wait for 1/2 of the Baud rate period
       lcall BaudDelayShort
       ret

; =============================================================================
; subroutine BaudDelayShort
;
; Suspends the program for a constant period
;
; the following machine cycles are computed assuming
; a 11.059 Mhz crystal.  the actual number of cycles
; in the loop is a few less to compensate for the
; rest of the code executio;
; Baud Rate     Machine Cycles     (1/2) number of cycles
;
;    1200            767                 383
;    2400            386                 193
;    4800            192                  96
;    9600             96                  48
;   19200             48                  24
;
;  input    : none
;  output   : none
;  destroys : nothing - acc
; -----------------------------------------------------------------------------
BaudDelayShort:
       push  acc         ; save acc on stack

; --- the constants below are a few less then the ---
; --- number of cycles to compensate for the --------
; --- execution of the rest of the code -------------

;       mov   a, #120     ; (383/3) for 1200 Baud
       mov   a, #60      ; (193/3) for 2400 Baud
;       mov   a, #27      ;  (96/3) for 4800 Baud
;       mov   a, #11      ;  (48/3) for 9600 Baud
BD0:   dec   a           ; takes 1 cycle
       jnz   BD0         ; takes 2 cycles

       pop   acc         ; restore acc
       ret

; =============================================================================
; subroutine Bin5 - RS-485 Serial Block Input
;
; This routine reads a block of 5 bytes from the
; RS-485 driver and saves in InBuffer.
;
; This routine must be called after the start bit is
; detected.  If less than 5 bytes are received and
; the routine times out, a long jump is made to the
; beginning of the routine.  Sometimes it is a better
; idea to jump to 0 to repeat the RESET sequence.
;
;  input    : none
;  output   : 5 bytes in InBuffer
;  destroys : R0, R1, R2, acc
; -----------------------------------------------------------------------------
Bin5:
       mov   dptr, #InBuffer ; the 5 bytes will be stored here
       mov   R0, #5          ; byte count
BI5_1:
       lcall Sin             ; receive next byte
       push  acc             ; save data byte
       lcall prsphx          ; print byte received on host (RS-232)
       pop   acc             ; recall data byte
       movx  @dptr, a        ; save in buffer
       inc   dptr            ; update buffer pointer
       djnz  R0, BI5_2       ; update byte counter, if more bytes...
                             ; branch to wait for next start bit
       lcall crlf            ; else send a new line to the host (RS-232)
       ret                   ; and return
BI5_2:
       mov   r1, #0          ; waiting for the start bit begins here...
       mov   r2, #80h        ; set up loop counts for time out
BI5_3:
       jnb   Data485, BI5_1  ; branch to read next data byte if start bit
       djnz  r1, BI5_3       ; else update loop counts and ...
       djnz  r2, BI5_3       ; ...repeat
       mov   a, #'.'         ; if time out...
       lcall sndchr          ; ...send a '.' to host (RS-232)...
       lcall crlf            ; ...and a new line

       ljmp ProgStart        ; long jump to the beginning of the program
                             ; to recover from the error
                             ; << modify to jump to 0 (RESET) if prefered >>
