; -----------------------------------------------------------------------------
; R-SLAVE1.asm      Last Update:06/28/92
;
; RS-485 network demo program for the prototyping board
;
; This routine emulates a PC network slave node.
; This routine illustrates the skeletal network slave code
; which will partially work with NETDEMO.exe since some of
; its command service routines do not respond back to the
; network master node.
;
; Run R-MASTER on one board and R-SLAVE1 on other node(s).
; Also try running R-SLAVE1 with NETDEMO.exe
;
; The address of the slave node is defined by an EQU
; pseudo-op as constant HERE.  Each slave node on the network
; must have a distinct address to avoid conflicts.
;
;
; Courtesy Rigel Corporation
;
; =============================================================================
;       m a i n    p r o g r a m
; -----------------------------------------------------------------------------
; constants
  HERE   equ 2     ; address of this node
  OKcmd  equ 0

; -----------------------------------------------------------------------------
; port bits

  Dir485  equ P1.3  ; Dir485=0 for input, 1 for output
  Data485 equ P1.2  ; Data input/output
; -----------------------------------------------------------------------------
; registers
  OutByte equ 7fh   ; saves the digital output port status
; -----------------------------------------------------------------------------

       org   8000h
ProgStart:                    ; program starts here
       clr   RI               ; flush serial input
       clr   Dir485           ; setup for input

loop0:
       jnb   RI, loop1        ; see if a key is pending (from the RS-232 host)
       lcall getchr           ; if so get char...
       cjne  a, #3, loop1     ; ... if not ctl-C branch to loop1...
       ljmp  0                ; ... else return to monitor via RESET
loop1:
       jb    Data485, loop0   ; see if a start bit is on the network
                              ; if not repeat (branch to loop0)

                              ; if start bit is detected on the network ...
       lcall Bin5             ; ...read message of 5 bytes.  Bin5 places ...
                              ; ... the 5 bytes in external memory ...
                              ; ... starting at address defined by InBuffer

       mov   dptr, #InBuffer  ; point to the start of message block
       movx  a, @dptr         ; read source address into acc

       inc   dptr             ; point to next message byte
       movx  a, @dptr         ; read destination address into acc
       cjne  a, #HERE, loop0  ; done if not this address, repeat

       inc   dptr             ; else point to next message byte
       movx  a, @dptr         ; read command into acc
       push  acc              ; save command
       clr   C                ; clear carry flag, setup for subtraction
       subb  a, #5            ; command-5
       pop   acc              ; recall command
       jnc   loop0            ; if no barrow command>4, (invalid) done, repeat

                              ; if command is valid (0<command<5) then...
                              ; ... jump to the corresponding command ...
                              ; ...service routine.  The service routine ...
                              ; ... addresses are stored in the table ...
                              ; ...labeled CmdTable.

       mov   dptr, #CmdTable  ; command table base
       rl    a                ; multiply acc by 2
       mov   b, a             ; save acc in b register
       inc   a                ; get ready to read high byte
       movc  a, @a+dptr       ; get high byte - low address of service routine
       push  acc              ; push command service routine low addr on stack
       mov   a, b             ; recall acc to...
       movc  a, @a+dptr       ; ... read command service routine high address
       push  acc              ; push command service routine high addr on stack
       ret                    ; ljump to command service routine by a return
                              ; Note: return pops 2 bytes from stack into...
                              ; ... program counter, next instruction...
                              ;... fetched is thus the command service routine
CmdTable:
       dw    SlaveSendDigital    ; Cmd 0
       dw    SlaveSendAnalog     ; Cmd 1
       dw    SlaveReceiveDigital ; Cmd 2
       dw    SlaveEchoDigital    ; Cmd 3
       dw    Acknowledge         ; Cmd 4


; --- Command Service Routines ------------------------------------------------
; -----------------------------------------------------------------------------
SlaveSendDigital:
       lcall print
       db   "Slave_Send_Digital message received", 0ah, 0dh, 0

       mov   a, #HERE ; source address
       lcall Sout     ; send message byte

       mov   a, #0    ; master's address
       lcall Sout     ; send message byte

       mov   a, #0    ; echo  Slave_Send_Digital command
       lcall Sout     ; send message byte

       mov   a, #0aah ; send data byte
       lcall Sout     ; send message byte

       clr   a        ; dummy byte is 0
       lcall Sout     ; send dummy byte

       ljmp  loop0
; -----------------------------------------------------------------------------
SlaveReceiveDigital:
       lcall print
       db   "Slave_Receive_Digital message received..."
       db   "data byte is ",0

       mov   dptr, #InBuffer
       inc   dptr
       inc   dptr
       inc   dptr
       movx  a, @dptr   ; get data byte
       mov   OutByte, a ; save data byte
       push  acc        ; save data byte for processing later
       lcall prthex
       lcall crlf

       pop   acc       ; recall data byte
       mov   C, acc.0  ; data bit 0 goes to P1.0
       cpl   C         ; complement C so that a 1 lights LED
       mov   P1.0, C
       mov   C, acc.1  ; data bit 1 goes to P1.1
       cpl   C         ; complement C so that a 1 lights LED
       mov   P1.1, C

       ljmp  loop0
; -----------------------------------------------------------------------------
SlaveEchoDigital:
       lcall print
       db   "Slave_Echo_Digital message received... data byte sent: ", 0

       mov   a, #HERE ; source address
       lcall Sout     ; send message byte
       mov   a, #0    ; master's address
       lcall Sout     ; send message byte
       mov   a, #3    ; send echo command
       lcall Sout     ; send message byte

       clr   a        ; set up acc for data byte
       mov   C, P1.4  ; place push button values
       cpl   C        ; send 1 if button pressed
       mov   acc.0, C
       mov   C, P1.5  ; now push button 1
       cpl   C
       mov   acc.1, C

       mov   b, OutByte ; recall output byte
       Mov   C, b.0     ; place LED 0 value
       mov   acc.2, C
       mov   C, b.1     ; now LED 1
       mov   acc.3, C
       push  acc
       lcall Sout     ; send data byte
       pop   acc
       lcall prthex
       lcall crlf

       clr   a        ; dummy byte is 0
       lcall Sout     ; send dummy byte

       ljmp  loop0
; -----------------------------------------------------------------------------
SlaveSendAnalog:
       lcall print
       db   "Slave_Send_Analog message received", 0ah, 0dh, 0
       ljmp  loop0
; -----------------------------------------------------------------------------
Acknowledge:
       lcall print
       db   "Acknowledge message received", 0ah, 0dh, 0
       mov   a, #0     ; master's address
       lcall Sout      ; send address
       mov   a, #HERE  ; this node's address
       lcall Sout      ; send address
       mov   a, #OKcmd ; OK command
       lcall Sout      ; send command
       clr   a         ; dummy data byte
       lcall Sout      ; send data byte
       ljmp  loop0
; -----------------------------------------------------------------------------
; =============================================================================
; include files

#include r-util.inc
#include r-net.inc

; =============================================================================
;
; end of program
